import React, {
  useState,
  useCallback,
  useEffect,
} from 'react';
import ReactDOM from 'react-dom';
import {
  csv,
  scaleLinear,
  scaleTime,
  min,
  max,
  axisLeft,
  axisBottom,
  timeFormat,
  extent,
  histogram as bin,
  timeMonths,
  sum
} from 'd3';
import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';

const height = 500;
const width = 960;
const centerX = width / 2;
const centerY = height / 2;
const margin = {
  top: 70,
  bottom: 70,
  left: 100,
  right: 70,
};
const tickOffset = 5;
const xAxisLabelOffset = 50;
const yAxisLabelOffset = -150;
const innerHeight = height - margin.top - margin.bottom;
const innerWidth = width - margin.left - margin.right;
const xValue = (d) => d.time;
const yValue = (d) => d.total;

const xAxisLabel = 'Month';
const yAxisLabel = 'Total Dead and Missing';

const xAxisTickFormat = timeFormat('%m/%d/%y');

const App = () => {
  const data = useData();

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const binnedData = bin()
    .value(xValue)
    .domain(xScale.domain())
    .thresholds(timeMonths(xScale.domain()[0], xScale.domain()[1]))
  	(data)
  	.map(array => ({
    	y: sum(array, yValue),
      x0: array.x0,
      x1: array.x1,
    }));
  
   const yScale = scaleLinear()
    .domain([0, max(binnedData, d => d.y)])
    .range([innerHeight, 0])
    .nice();
  
  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        <AxisBottom
          xScale={xScale}
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
          tickOffset={tickOffset}
        />
        <AxisLeft
          yScale={yScale}
          innerWidth={innerWidth}
          tickOffset={tickOffset}
        />
        <text
          x={innerWidth / 2}
          style={{ textAnchor: 'middle' }}
          y={innerHeight + xAxisLabelOffset}
          className="axis-label"
        >
          {xAxisLabel}
        </text>
        <text
          x={yAxisLabelOffset}
          style={{ textAnchor: 'middle' }}
          y={-innerHeight / 2 + 120}
          className="axis-label"
          transform="rotate(-90)"
        >
          {yAxisLabel}
        </text>
        <Marks
          xScale={xScale}
          yScale={yScale}
          binnedData={binnedData}
          tooltipFormat={xAxisTickFormat}
          innerHeight={innerHeight}
        />
      </g>
    </svg>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
